@@ -115,11 +115,8 @@ QPDF_Stream::QPDF_Stream(
     qpdf_offset_t offset,
     size_t length) :
     QPDFValue(::ot_stream, "stream"),
-    qpdf(qpdf),
-    og(og),
     filter_on_write(true),
     stream_dict(stream_dict),
-    offset(offset),
     length(length)
 {
     if (!stream_dict.isDictionary()) {
@@ -128,6 +125,7 @@ QPDF_Stream::QPDF_Stream(
     }
     setDescription(
         qpdf, qpdf->getFilename() + ", stream object " + og.unparse(' '));
+    this->parsed_offset = offset;
 }
 
 std::shared_ptr<QPDFObject>
@@ -142,9 +140,10 @@ QPDF_Stream::create(
 }
 
 std::shared_ptr<QPDFObject>
-QPDF_Stream::shallowCopy()
+QPDF_Stream::copy(bool shallow)
 {
-    throw std::logic_error("stream objects cannot be cloned");
+    QTC::TC("qpdf", "QPDF_Stream ERR shallow copy stream");
+    throw std::runtime_error("stream objects cannot be cloned");
 }
 
 void
@@ -174,17 +173,6 @@ QPDF_Stream::disconnect()
     QPDFObjectHandle::DisconnectAccess::disconnect(this->stream_dict);
 }
 
-void
-QPDF_Stream::setObjGen(QPDFObjGen const& og)
-{
-    if (this->og.isIndirect()) {
-        throw std::logic_error(
-            "attempt to set object ID and generation of a stream"
-            " that already has them");
-    }
-    this->og = og;
-}
-
 std::string
 QPDF_Stream::unparse()
 {
@@ -213,14 +201,15 @@ QPDF_Stream::getStreamJSON(
     case qpdf_sj_none:
     case qpdf_sj_inline:
         if (p != nullptr) {
-            throw std::logic_error("QPDF_Stream::getStreamJSON: pipline should "
-                                   "only be suppiled json_data is file");
+            throw std::logic_error(
+                "QPDF_Stream::getStreamJSON: pipeline should "
+                "only be supplied when json_data is file");
         }
         break;
     case qpdf_sj_file:
         if (p == nullptr) {
-            throw std::logic_error("QPDF_Stream::getStreamJSON: pipline must "
-                                   "be be suppiled json_data is file");
+            throw std::logic_error("QPDF_Stream::getStreamJSON: pipeline must "
+                                   "be supplied when json_data is file");
         }
         if (data_filename.empty()) {
             throw std::logic_error("QPDF_Stream::getStreamJSON: data_filename "
@@ -323,12 +312,6 @@ QPDF_Stream::isDataModified() const
     return (!this->token_filters.empty());
 }
 
-qpdf_offset_t
-QPDF_Stream::getOffset() const
-{
-    return this->offset;
-}
-
 size_t
 QPDF_Stream::getLength() const
 {
@@ -358,7 +341,7 @@ QPDF_Stream::getStreamData(qpdf_stream_decode_level_e decode_level)
             qpdf_e_unsupported,
             qpdf->getFilename(),
             "",
-            this->offset,
+            this->parsed_offset,
             "getStreamData called on unfilterable stream");
     }
     QTC::TC("qpdf", "QPDF_Stream getStreamData");
@@ -374,7 +357,7 @@ QPDF_Stream::getRawStreamData()
             qpdf_e_unsupported,
             qpdf->getFilename(),
             "",
-            this->offset,
+            this->parsed_offset,
             "error getting raw stream data");
     }
     QTC::TC("qpdf", "QPDF_Stream getRawStreamData");
@@ -416,10 +399,7 @@ QPDF_Stream::filterable(
 
     if (!filters_okay) {
         QTC::TC("qpdf", "QPDF_Stream invalid filter");
-        warn(
-            qpdf_e_damaged_pdf,
-            this->offset,
-            "stream filter type is not name or array");
+        warn("stream filter type is not name or array");
         return false;
     }
 
@@ -467,11 +447,7 @@ QPDF_Stream::filterable(
     // one case of a file whose /DecodeParms was [ << >> ] when
     // /Filters was empty has been seen in the wild.
     if ((filters.size() != 0) && (decode_parms.size() != filters.size())) {
-        warn(
-            qpdf_e_damaged_pdf,
-            this->offset,
-            "stream /DecodeParms length is"
-            " inconsistent with filters");
+        warn("stream /DecodeParms length is inconsistent with filters");
         filterable = false;
     }
 
@@ -583,9 +559,8 @@ QPDF_Stream::pipeStreamData(
             }
             Pl_Flate* flate = dynamic_cast<Pl_Flate*>(pipeline);
             if (flate != nullptr) {
-                flate->setWarnCallback([this](char const* msg, int code) {
-                    warn(qpdf_e_damaged_pdf, this->offset, msg);
-                });
+                flate->setWarnCallback(
+                    [this](char const* msg, int code) { warn(msg); });
             }
         }
     }
@@ -627,7 +602,7 @@ QPDF_Stream::pipeStreamData(
             this->stream_dict.replaceKey(
                 "/Length", QPDFObjectHandle::newInteger(actual_length));
         }
-    } else if (this->offset == 0) {
+    } else if (this->parsed_offset == 0) {
         QTC::TC("qpdf", "QPDF_Stream pipe no stream data");
         throw std::logic_error("pipeStreamData called for stream with no data");
     } else {
@@ -635,7 +610,7 @@ QPDF_Stream::pipeStreamData(
         if (!QPDF::Pipe::pipeStreamData(
                 this->qpdf,
                 og,
-                this->offset,
+                this->parsed_offset,
                 this->length,
                 this->stream_dict,
                 pipeline,
@@ -648,28 +623,17 @@ QPDF_Stream::pipeStreamData(
 
     if (filter && (!suppress_warnings) && normalizer.get() &&
         normalizer->anyBadTokens()) {
-        warn(
-            qpdf_e_damaged_pdf,
-            this->offset,
-            "content normalization encountered bad tokens");
+        warn("content normalization encountered bad tokens");
         if (normalizer->lastTokenWasBad()) {
             QTC::TC("qpdf", "QPDF_Stream bad token at end during normalize");
-            warn(
-                qpdf_e_damaged_pdf,
-                this->offset,
-                "normalized content ended with a bad token;"
-                " you may be able to resolve this by"
-                " coalescing content streams in combination"
-                " with normalizing content. From the command"
-                " line, specify --coalesce-contents");
+            warn("normalized content ended with a bad token; you may be able "
+                 "to resolve this by coalescing content streams in combination "
+                 "with normalizing content. From the command line, specify "
+                 "--coalesce-contents");
         }
-        warn(
-            qpdf_e_damaged_pdf,
-            this->offset,
-            "Resulting stream data may be corrupted but is"
-            " may still useful for manual inspection."
-            " For more information on this warning, search"
-            " for content normalization in the manual.");
+        warn("Resulting stream data may be corrupted but is may still useful "
+             "for manual inspection. For more information on this warning, "
+             "search for content normalization in the manual.");
     }
 
     return success;
@@ -734,10 +698,7 @@ QPDF_Stream::replaceDict(QPDFObjectHandle const& new_dict)
 }
 
 void
-QPDF_Stream::warn(
-    qpdf_error_code_e error_code,
-    qpdf_offset_t offset,
-    std::string const& message)
+QPDF_Stream::warn(std::string const& message)
 {
-    this->qpdf->warn(error_code, "", offset, message);
+    this->qpdf->warn(qpdf_e_damaged_pdf, "", this->parsed_offset, message);
 }
